@@ -15,7 +15,7 @@ from typing import Tuple, Union
 # choose font - can be deactivated
 from matplotlib import rcParams
 from oscovida.data_sources import base_url, hungary_data, jhu_population_url, rki_data, rki_population_url
-from oscovida.plotting_helpers import align_twinx_ticks, has_twin, limit_to_smoothed
+from oscovida.plotting_helpers import align_twinx_ticks, cut_dates, has_twin, limit_to_smoothed
 
 rcParams['font.family'] = 'sans-serif'
 rcParams['font.sans-serif'] = ['Inconsolata']
@@ -268,7 +268,7 @@ def fetch_data_germany_last_execution():
 
 
 @joblib_memory.cache
-def fetch_data_germany(include_last_day=True, filter_goettingen_alt=True) -> pd.DataFrame:
+def fetch_data_germany(include_last_day=True) -> pd.DataFrame:
     """Fetch data for Germany from Robert Koch institute and return as a pandas
     DataFrame with the `Meldedatum` as the index.
 
@@ -279,10 +279,6 @@ def fetch_data_germany(include_last_day=True, filter_goettingen_alt=True) -> pd.
     the retrieved data sets (see reasoning below in source), as the data is
     commonly update one day later with more accurate (and typically higher)
     numbers.
-
-    The `filter_goettingen_alt` option is only needed for a unit test
-    (test_corona.py::test_clean_data_germany_goettingen_alt_is_fluke).
-
     """
 
     # outdated: datasource = "https://opendata.arcgis.com/datasets/dd4580c810204019a7b8eb3e0b329dd6_0.csv"
@@ -329,14 +325,8 @@ def fetch_data_germany(include_last_day=True, filter_goettingen_alt=True) -> pd.
     else:
         cleaned = g2
 
-    # see documentation of clean_data_germany_goettingen_alt for explanation
-    if filter_goettingen_alt:
-        cleaned2 = clean_data_germany_remove_goettingen_alt(cleaned)
-    else:
-        cleaned2 = cleaned
-
     fetch_data_germany_last_execution()
-    return cleaned2
+    return cleaned
 
 
 def pad_cumulative_series_to_yesterday(series):
@@ -746,7 +736,9 @@ def plot_time_step(ax, series, style="-", labels=None, logscale=True):
     return ax
 
 
-def compute_daily_change(series):
+def compute_daily_change(series: pd.Series) -> Tuple[Tuple[pd.Series, str],
+                                                     Tuple[pd.Series, str],
+                                                     Tuple[pd.Series, str]]:
     """returns (change, smooth, smooth2)
 
     where 'change' is a tuple of (series, label)
@@ -773,13 +765,14 @@ def compute_daily_change(series):
     # Smoothing step 1: get rid of weekly cycles
     rolling_series = diff.rolling(7, center=True,
                                   win_type=None,
-                                  min_periods=1).mean()
+                                  min_periods=7).mean()
     smooth = rolling_series, smooth_label
 
     # extra smoothing for better visual effects
-    rolling_series2 = rolling_series.rolling(9, center=True,
+    rolling_series2 = rolling_series.rolling(7, center=True,
                                              win_type='gaussian',
-                                             min_periods=1).mean(std=3)
+                                             min_periods=7).mean(std=3)
+
     # extra smooth curve
     smooth2_label = "Smoothed " + smooth_label
     # shorter description
@@ -791,7 +784,8 @@ def compute_daily_change(series):
 
 
 def plot_daily_change(ax, series: pd.Series, color: str, labels: Tuple[str, str] = None,
-                      country: str = None, region: str = None, subregion: str = None):
+                      country: str = None, region: str = None, subregion: str = None,
+                      dates: str = None, weeks: int = 0):
     """Given a series of data and matplotlib axis ax, plot the
     - difference in the series data from day to day as bars and plot a smooth
     - line to show the overall development
@@ -810,17 +804,22 @@ def plot_daily_change(ax, series: pd.Series, color: str, labels: Tuple[str, str]
         region_label, type_label = labels
     ax_label = region_label + " new " + type_label
     (change, change_label), _, (smooth2, smooth2_label) = compute_daily_change(series)
+    if dates:
+        change = cut_dates(change, dates)
+        smooth2 = cut_dates(smooth2, dates)
+    if weeks:
+        change = change[-7 * weeks:]
+        smooth2 = smooth2[-7 * weeks:]
 
     if country and (region or subregion):
         habitants = population(country=country, region=region, subregion=subregion)
     else:
         habitants = population(region_label)
-    if region_label and habitants and max(change) > 0:
+    if habitants and max(change) > 0:
         # create another Y-axis on the right hand side
         # unfortunately there's no simple way of swapping the axes,
         # therefore we define normalised axis first
         ax2 = ax
-        ax2.grid(None)  # disabling the second grid
         # this is just to be sure that we plot the same graph (please leave it commented in the production):
         # ax2.plot(smooth2.index, smooth2.values * 1E5 / habitants, color='green')
         ax2.set_ylim((0, max(change) * 1E5 / habitants))
@@ -836,6 +835,13 @@ def plot_daily_change(ax, series: pd.Series, color: str, labels: Tuple[str, str]
 
         ax1.plot(smooth2.index, smooth2.values, color=color,
                  label=ax_label + " " + smooth2_label, linewidth=LW)
+
+        # uncertain graph on the right
+        if not dates:
+            fortnight = series[-14:].diff().dropna()
+            uncert = fortnight.rolling(14, center=True, win_type='gaussian', min_periods=3).mean(std=4)
+            ax1.plot(uncert.index[-7:], uncert.values[-7:], color=color, linestyle='dashed', linewidth=LW, alpha=0.7)
+
         ax1.legend()
         ax1.set_ylabel('daily change')
 
@@ -1016,6 +1022,12 @@ def compute_growth_factor(series):
     # division by zero may lead to np.inf in the data: get rid of that
     f.replace(np.inf, np.nan, inplace=True)  # seems not to affect plot
 
+    # pct_change computed between two values that are NaN each, returns 0
+    # (which should be interpreted as: no change from yesterday's NaN to
+    #  today's NaN. However, the way we use it, this gives misleading values. )
+    # Let's take out those values from f, where the input data was NaN:
+    f[smooth.isna()]=np.nan
+
     # Compute smoother version for line in plots
     f_smoothed = f.rolling(7, center=True, win_type='gaussian', min_periods=3).mean(std=2)
     smooth_label = f"Gaussian window (stddev=2 days)"
@@ -1124,7 +1136,7 @@ def min_max_in_past_n_days(series, n, at_least = [0.75, 1.25], alert=[0.1, 100],
 
 
 def plot_reproduction_number(ax, series, color_g='C1', color_R='C4',
-                             yscale_days=28, max_yscale=10,
+                             yscale_days=28, max_yscale=10, smoothing=0,
                              labels=None):
     """
     - series is expected to be time series of cases or deaths
@@ -1133,6 +1145,15 @@ def plot_reproduction_number(ax, series, color_g='C1', color_R='C4',
     - color_g is the colour for the growth factor
     - color_R is the colour for the reproduction number
 
+    R number is calculated based on a 7-day average of the daily changes.
+    See details in compute_R().
+
+    By default the resulting data is not smoothed further. If desired,
+    for visual purposes, the parameter `smoothing` can be given the number
+    of days over which the computer R-values are averaged out (using a Gaussian
+    window with a width of 3 days). A number of 5 or 7 gives smooth results.
+    However, the plotted data then stops 5/2 (=2 days) or 7/2 (=3) days earlier.
+
     See plot_time_step for documentation on other parameters.
     """
 
@@ -1141,15 +1162,15 @@ def plot_reproduction_number(ax, series, color_g='C1', color_R='C4',
     else:
         region, label = labels
 
-     # get smooth data for growth factor from plot 1 to base this plot on
-    (f, f_label) , (f_smoothed, smoothed_label) = compute_growth_factor(series)
+    # get smooth data for growth factor from plot 1 to base this plot on
+    (f, f_label), (f_smoothed, smoothed_label) = compute_growth_factor(series)
 
     label_ = region + " " + label + " daily growth factor " + f_label
     ax.plot(f.index, f.values, 'o', color=color_g, alpha=0.3, label=label_)
 
     label_ = region + " " + label + " daily growth factor " + smoothed_label
-    ax.plot(f_smoothed.index, f_smoothed.values, '-', color=color_g, label=label_, linewidth=LW,
-            alpha=0.7)
+    ax.plot(f_smoothed.index, f_smoothed.values, '-', color=color_g,
+            label=label_, linewidth=LW, alpha=0.7)
 
 
     # # data for computation or R
@@ -1157,9 +1178,13 @@ def plot_reproduction_number(ax, series, color_g='C1', color_R='C4',
     (change, change_label) , (smooth, smooth_label), \
         (smooth2, smooth2_label) = compute_daily_change(series)
 
-    # we only use one return value:
-    smooth_diff = smooth2
+    # we only use one return value (the 7-day average)
+    smooth_diff = smooth
     R = compute_R(smooth_diff)
+    # do some additional smoothing
+    if smoothing != 0:
+        R = R.rolling(smoothing, win_type='gaussian', center=True,
+                      min_periods=smoothing).mean(std=3)
     ax.plot(R.index, R, "-", color=color_R,
             label=region + f" estimated R (using {label})",
             linewidth=4.5, alpha=1)
@@ -1283,35 +1308,29 @@ def get_country_data(country: str, region: str = None, subregion: str = None, ve
         print(f"get_country_data: deaths[{len_deaths1}] -> [{len_deaths2}]")
     return c, d
 
-#######################
 
-def day0atleast(v0, series):
+def day0atleast(v0: int, series: pd.Series) -> pd.Series:
     try:
         day0 = series[series > v0].index[0]
     except IndexError:  # means no days found for which series.values > v0
-        # print(f"Haven't found value > {v0} is Series {series.name}")
-        result = pd.Series(dtype=object)
+        # print(f"Haven't found value > {v0} in {series.name}")
+        result = pd.Series(dtype=object)    # return an empty object
         return result
-
     # compute timedelta
     timedelta = series.index - day0
     # convert to int as index
     t = pd.to_numeric(timedelta.astype("timedelta64[D]").astype(int))
     # Assemble new series
     result = pd.Series(index=t, data=series.values)
-
     return result
 
 
-
-
 def align_sets_at(v0, df):
     """Accepts data frame, and aligns so that all entries close to v0 are on the same row.
 
     Returns new dataframe with integer index (representing days after v0).
     """
     res = pd.DataFrame()
-
     for col in df.columns:
         # res[col] = day0for(v0, df[col])
         series = day0atleast(v0, df[col])
@@ -1383,6 +1402,8 @@ def plot_logdiff_time(ax, df, xaxislabel=None, yaxislabel=None, style="", labels
 
                 # Add country/region name as text next to last data point of the line:
                 ax.annotate(col, xy=(x + labeloffset, y), textcoords='data')
+            else:
+                ax.legend()
     ax.set_ylabel(yaxislabel)
     ax.set_xlabel(xaxislabel)
     ax.set_yscale('log')
@@ -1390,7 +1411,7 @@ def plot_logdiff_time(ax, df, xaxislabel=None, yaxislabel=None, style="", labels
     # from https://stackoverflow.com/questions/21920233/matplotlib-log-scale-tick-label-number-formatting/33213196
     ax.yaxis.set_major_formatter(FuncFormatter(lambda y, _: '{:g}'.format(y)))
     # ax.set_xscale('log')    # also interesting
-    if isinstance(df.index[0], int):
+    if isinstance(df.index[0], (int, np.integer)):
         ax.set_ylim(bottom=set_y_axis_limit(df, v0))
         ax.set_xlim(left=-1)  #ax.set_xlim(-1, df.index.max())
     else:
@@ -1545,7 +1566,7 @@ def get_compare_data_hungary(region, compare_with_local: list, rolling=7):
     return df_c, None
 
 
-def make_compare_plot_germany(region_subregion,
+def make_compare_plot_germany(region=None, subregion=None,
                               compare_with=[],  # "China", "Italy", "Germany"],
                               compare_with_local=['Bayern',
                                                   'Berlin', 'Bremen',
@@ -1555,7 +1576,6 @@ def make_compare_plot_germany(region_subregion,
                               v0c=10, v0d=1, normalise=True,
                               weeks=0, dates=None):
     rolling = 7
-    region, subregion = unpack_region_subregion(region_subregion)
     df_c1, df_d1 = get_compare_data_germany((region, subregion), compare_with_local, rolling=rolling)
     df_c2, df_d2 = get_compare_data(compare_with, rolling=rolling)
 
@@ -1569,15 +1589,10 @@ def make_compare_plot_germany(region_subregion,
     kwargs_c, kwargs_d = {}, {}
 
     if dates and weeks == 0:
-        try:
-            date_start, date_end = dates.split(':')
-            res_c = df_c[date_start:date_end]
-            res_d = df_d[date_start:date_end]
-            kwargs_c.update({'labels': False})
-            kwargs_d.update({'labels': False})
-        except ValueError:
-            raise ValueError(f"`dates` are not a valid time range, try something "
-                             f"like dates='{df_c.index[0].date()}:{df_c.index[-1].date()}'")
+        res_c = cut_dates(df_c, dates)
+        res_d = cut_dates(df_d, dates)
+        kwargs_c.update({'labels': False})
+        kwargs_d.update({'labels': False})
     elif dates and weeks:
         raise ValueError("`dates` and `weeks` cannot be used together")
     elif weeks > 0:
@@ -1689,7 +1704,7 @@ def plot_no_data_available(ax, mimic_subplot, text):
 
 def overview(country: str, region: str = None, subregion: str = None,
              savefig: bool = False, dates: str = None,
-             weeks: int = 0) -> Tuple[plt.axes, pd.Series, pd.Series]:
+             weeks: int = 0, data: Tuple[pd.Series, pd.Series] = None) -> Tuple[plt.axes, pd.Series, pd.Series]:
     """The `overview` function provides 6 graphs for the region:
 
     0) the total cumulative number of cases and deaths
@@ -1704,38 +1719,40 @@ def overview(country: str, region: str = None, subregion: str = None,
 
     Returns: subplots, cases (pandas Series), deaths (pandas Series)
     """
-    c, d = get_country_data(country, region=region, subregion=subregion)
+    if data is None:
+        _c, _d = get_country_data(country, region=region, subregion=subregion)
+    else:
+        _c, _d = data
+        assert isinstance(_c.index[0], pd.Timestamp), f"The index of 'cases' is not of type `Timestamp`, " \
+                                                     f"try to use `index=pd.DatetimeIndex(dates)`"
+        assert isinstance(_d.index[0], pd.Timestamp), f"The index of 'deaths' is not of type `Timestamp`, " \
+                                                     f"try to use `index=pd.DatetimeIndex(dates)`"
+
     region_label = get_region_label(country, region=region, subregion=subregion)
     fig, axes = plt.subplots(6, 1, figsize=(10, 15), sharex=False)
     if dates and weeks == 0:
-        try:
-            date_start, date_end = dates.split(':')
-            c = c[date_start:date_end]
-        except ValueError:
-            raise ValueError(f"`dates` are not a valid time range, try something "
-                             f"like dates='{c.index[0].date()}:{c.index[-1].date()}'")
+        c = cut_dates(_c, dates)
     elif dates and weeks:
         raise ValueError("`dates` and `weeks` cannot be used together")
     else:
-        c = c[- weeks * 7:]
+        c = _c[- weeks * 7:]
     plot_time_step(ax=axes[0], series=c, style="-C1", labels=(region_label, "cases"))
-    plot_daily_change(ax=axes[1], series=c, color="C1", labels=(region_label, "cases"),
-                      country=country, region=region, subregion=subregion)
+    plot_daily_change(ax=axes[1], series=_c, color="C1", labels=(region_label, "cases"),
+                      country=country, region=region, subregion=subregion, dates=dates, weeks=weeks)
     # data cleaning
     if country == "Spain":   # https://github.com/oscovida/oscovida/issues/44
         axes[1].set_ylim(bottom=0)
     plot_reproduction_number(axes[3], series=c, color_g="C1", color_R="C5", labels=(region_label, "cases"))
     ax_dt_c = axes[5]
     plot_doubling_time(ax_dt_c, series=c, color="C1", labels=(region_label, "cases"))
-    if d is not None:
+    if _d is not None:
         if dates and weeks == 0:
-            date_start, date_end = dates.split(':')
-            d = d[date_start:date_end]
+            d = cut_dates(_d, dates)
         else:
-            d = d[- weeks * 7:]
+            d = _d[- weeks * 7:]
         plot_time_step(ax=axes[0], series=d, style="-C0", labels=(region_label, "deaths"))
-        plot_daily_change(ax=axes[2], series=d, color="C0", labels=(region_label, "deaths"),
-                          country=country, region=region, subregion=subregion)
+        plot_daily_change(ax=axes[2], series=_d, color="C0", labels=(region_label, "deaths"),
+                          country=country, region=region, subregion=subregion, dates=dates, weeks=weeks)
         plot_reproduction_number(axes[4], series=d, color_g="C0", color_R="C4", labels=(region_label, "deaths"))
         problems = ("no data in reduced data set", "Cannot compute smooth ratio")
         if compute_doubling_time(d)[0][1] not in problems:
@@ -1755,15 +1772,10 @@ def overview(country: str, region: str = None, subregion: str = None,
             axes[5].legend(*labels)
         else:   # just create a legend as is
             axes[5].legend()
-    if d is None:
+    if _d is None:
+        d = _d
         plot_no_data_available(axes[2], mimic_subplot=axes[1], text='daily change in deaths')
         plot_no_data_available(axes[4], mimic_subplot=axes[3], text='R & growth factor (based on deaths)')
-        # axes[2].set_visible(False)
-        # axes[4].set_visible(False)
-
-    # ax = axes[3]
-    # plot_growth_factor(ax, series=d, color="C0")
-    # plot_growth_factor(ax, series=c, color="C1")
 
     # enforce same x-axis on all plots
     for i in range(1, axes.shape[0]):
@@ -1789,7 +1801,8 @@ def overview(country: str, region: str = None, subregion: str = None,
 
 
 def compare_plot(country: str, region: str = None, subregion: str = None,
-                 savefig: bool = False, normalise: bool = True) -> Tuple[plt.axes, pd.Series, pd.Series]:
+                 savefig: bool = False, normalise: bool = True,
+                 dates: str = None) -> Tuple[plt.axes, pd.Series, pd.Series]:
     """ Create a pair of plots which show comparison of the region with other most suffering countries
     """
     c, d = get_country_data(country, region=region, subregion=subregion)
@@ -1807,7 +1820,8 @@ def compare_plot(country: str, region: str = None, subregion: str = None,
         # We thus compare only against those Laender, that are in the data set:
         # germany = fetch_data_germany()
         # laender = list(germany['Bundesland'].drop_duplicates().sort_values())
-        axes_compare, res_c, red_d = make_compare_plot_germany((region, subregion))
+        axes_compare, res_c, red_d = make_compare_plot_germany(region=region, subregion=subregion, normalise=normalise,
+                                                               dates=dates)
     elif country == "US" and region is not None:
         # skip comparison plot for the US states at the moment
         return None, c, d
@@ -1837,66 +1851,3 @@ def get_cases_last_week(cases):
     # last week is difference between last value, and the one 7 days before
     cases_last_week = c2[-1] - c2[-8]
     return cases_last_week
-
-
-
-def clean_data_germany_remove_goettingen_alt(germany_data, debug=False):
-    """New 16 September 2020.
-
-    This function removes all data rows that are for the region "LK Göttingen
-    (alt)".
-
-    Some diagnostic output is printed, if `debug=True` is used.
-
-
-    Context:
-
-    Some tests failed because we have a new Landkreis in the data from the RKI,
-    which is called "LK Göttingen (alt)". As of 16 September 2020, there is
-    only one entry. I speculate that this is an error and will disappear over
-    time. The proposed solution is to remove this line from the RKI data, and
-    to only do this if there is at most one row for "LK Göttingen (alt)".
-
-    Here is some debug code:
-
-    germany_data = oscovida.fetch_data_germany()
-    # find all rows for 'LK Göttingen (alt)'
-    sel = germany_data['Landkreis'] == 'LK Göttingen (alt)'
-    print(f"Found {len(germany_data[sel])} rows for Göttingen (alt)")
-    print(germany_data[sel].T)
-
-    with output:
-
-      Found 1 rows for Göttingen (alt)
-      date                             2020-04-09
-      FID                                35939791
-      IdBundesland                              3
-      Bundesland                    Niedersachsen
-      Landkreis                LK Göttingen (alt)
-      Altersgruppe                        A35-A59
-      Geschlecht                                W
-      AnzahlFall                                1
-      AnzahlTodesfall                           0
-      Meldedatum              2020/04/09 00:00:00
-      IdLandkreis                            3159
-      Datenstand            16.09.2020, 00:00 Uhr
-      NeuerFall                                 0
-      NeuerTodesfall                           -9
-      Refdatum                2020/04/09 00:00:00
-      NeuGenesen                                0
-      AnzahlGenesen                             1
-      IstErkrankungsbeginn                      0
-      Altersgruppe2             Nicht übermittelt
-
-    """
-
-    # find all rows for 'LK Göttingen (alt)'
-    sel = germany_data['Landkreis'] == 'LK Göttingen (alt)'
-    if debug:
-        print(f"Found {len(germany_data[sel])} rows for Göttingen (alt)")
-        print(germany_data[sel].T)
-
-    # return all other rows
-    return germany_data[~sel]
-
-
